#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <pwd.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"
#include "master.xpm"

#define WMMON_VERSION "1.0smp"
#define DEFAULT 0
#define PORTAGE 1
#define TECRA   2

char	*ProgName;
char	wmfan_mask_bits[64*64];
int	wmfan_mask_width = 64;
int	wmfan_mask_height = 64;
int	method;

void usage(void);
void wmfan_routine(int, char **);

int GetMachineID(int *id)
{
	unsigned short lsb,msb;
	unsigned char fl1,fl2;

	asm ("movw $0xfe00,%%ax\n\t" \
		"movw $0x0070,%%bx\n\t" \
		"movl $0x000ffffa,%%edx\n\t" \
		"inb $0xb2,%%al\n\t" \
		"movw %%cx,%0\n\t" \
		"lahf\n\t" \
		"movb %%ah,%1\n\t" \
		"movw $0xfe00,%%ax\n\t" \
		"movw $0x0070,%%bx\n\t" \
		"movl $0x000ffffe,%%edx\n\t" \
		"inb $0xb2,%%al\n\t" \
		"movw %%cx,%2\n\t" \
		"lahf\n\t" \
		"movb %%ah,%3\n"
		:"=m" (lsb), "=m" (fl1), "=m" (msb), "=m" (fl2) \
		: \
		: "memory" );

		*id = (lsb & 0xff) + 0x100*(msb & 0xff);

		return ((fl1 & 0x01)==1) || ((fl2 & 0x01)==1);
}

void FanOn(int method)
{
	switch (method) {
		case PORTAGE:
			asm("cli\n\t" \
				"movb $0xbe,%al\n\t" \
				"outb %al,$0xe4\n\t" \
				"inb $0xe5,%al\n\t" \
				"andb $0xfe,%al\n\t" \
				"movb %al,%ah\n\t" \
				"movb $0xbe,%al\n\t" \
				"outb %al,$0xe4\n\t" \
				"movb %ah,%al\n\t" \
				"outb %al,$0xe5\n\t" \
				"sti");
			break;
		case TECRA:
			asm("cli\n\t" \
				"movw $0x00e4,%dx\n\t" \
				"movb $0xe0,%al\n\t" \
				"outb %al,%dx\n\t" \
				"incw %dx\n\t" \
				"inb %dx,%al\n\t" \
				"orw $0x0001,%ax\n\t" \
				"decw %dx\n\t" \
				"movb %al,%ah\n\t" \
				"movb $0xe0,%al\n\t" \
				"outw %ax,%dx\n\t" \
				"sti");
			break;
		default:
			asm ("movw $0xffff,%ax\n\t" \
				"movw $0x0004,%bx\n\t" \
				"movw $0x0001,%cx\n\t" \
				"inb $0xb2,%al");
			break;
		}

	return;
}

void FanOff(int method)
{
	switch (method) {
		case PORTAGE:
			asm("cli\n\t" \
				"movb $0xbe,%al\n\t" \
				"outb %al,$0xe4\n\t" \
				"inb $0xe5,%al\n\t" \
				"orb $0x01,%al\n\t" \
				"movb %al,%ah\n\t" \
				"movb $0xbe,%al\n\t" \
				"outb %al,$0xe4\n\t" \
				"movb %ah,%al\n\t" \
				"outb %al,$0xe5\n\t" \
				"sti");
			break;
		case TECRA:
			asm("cli\n\t" \
				"movw $0x00e4,%dx\n\t" \
				"movb $0xe0,%al\n\t" \
				"outb %al,%dx\n\t" \
				"incw %dx\n\t" \
				"inb %dx,%al\n\t" \
				"andw $0xfffe,%ax\n\t" \
				"decw %dx\n\t" \
				"movb %al,%ah\n\t" \
				"movb $0xe0,%al\n\t" \
				"outw %ax,%dx\n\t" \
				"sti");
			break;
		default:
			asm ("movw $0xffff,%ax\n\t" \
				"movw $0x0004,%bx\n\t" \
				"movw $0x0000,%cx\n\t" \
				"inb $0xb2,%al");
			break;
		}

	return;
}

int FanStatus(int method)
{
	unsigned short status;

	switch(method) {
		case PORTAGE:
			asm("cli\n\t" \
				"movb $0xbe,%%al\n\t" \
				"outb %%al,$0xe4\n\t" \
				"inb $0xe5,%%al\n\t" \
				"andw $0x0001,%%ax\n\t" \
				"movw %%ax,%0\n\t" \
				"sti\n"
 				:"=m" (status) : : "memory" );
				status = (status==0x00) ? 0x01:0x00;
			break;
		case TECRA:
			asm("cli\n\t" \
				"movw $0x00e4,%%dx\n\t" \
				"movb $0xe0,%%al\n\t" \
				"outb %%al,%%dx\n\t" \
				"incw %%dx\n\t" \
				"inb %%dx,%%al\n\t" \
				"andw $0x0001,%%ax\n\t"
				"movw %%ax,%0\n\t" \
				"sti\n"
				:"=m" (status) : : "memory" );
			break;
		default:
			asm ("pushl %%eax\n\t" \
				"pushl %%ebx\n\t" \
				"pushl %%ecx\n\t" \
				"movw $0xfefe,%%ax\n\t" \
				"movw $0x0004,%%bx\n\t" \
				"movw $0x0000,%%cx\n\t" \
				"inb $0xb2,%%al\n\t" \
				"andw $0x0001,%%cx\n\t" \
				"movw %%cx,%0\n\t" \
				"popl %%ecx\n\t" \
				"popl %%ebx\n\t" \
				"popl %%eax\n"
 				:"=m" (status) : : "memory" );
			break;
		}

	return (int) status;
}


int main(int argc, char *argv[]) {
 int i,id;
 
 if (iopl(3)) {
  printf("wmfan: can't get I/O permissions.\n");
  exit (1);
 }
 printf("we got this far\n");
 seteuid(getuid());
 setegid(getgid());
 
 method = DEFAULT;
 /* if (GetMachineID(&id)==0x00) {
  if (id==0xfccb)
   method = PORTAGE;
  else if (id==0xfccc)
   method = TECRA;
 } */
 printf("we got this far\n");
 
 ProgName = argv[0];
 if (strlen(ProgName) >= 5)
 ProgName += (strlen(ProgName) - 5);
	
 for (i=1; i<argc; i++) {
  char *arg = argv[i];
  if (*arg=='-') {
   switch (arg[1]) {
    case 'd' :
     if (strcmp(arg+1, "display")) {
      usage();
      exit(1);
     }
     break;
    default:
     usage();
     exit(0);
    break;
   }
  }
 }
 wmfan_routine(argc, argv);
 return(0);
}

void wmfan_routine(int argc, char **argv) {
 XEvent Event;
 int i=0;
 int manual=0;

 createXBMfromXPM(wmfan_mask_bits, master_xpm, wmfan_mask_width, wmfan_mask_height);
 openXwindow(argc, argv, master_xpm, wmfan_mask_bits, wmfan_mask_width, wmfan_mask_height);
 AddMouseRegion(0,9,41,14,50);
 AddMouseRegion(1,9,46,14,55);


 
 // main loop
 while (1) {
  RedrawWindow();
  while (XPending(display)) {
   XNextEvent(display, &Event);
   switch (Event.type) {
   case Expose:
    RedrawWindow();
    break;
   case DestroyNotify:
    XCloseDisplay(display);
    exit(0);
    break;
   case ButtonPress:
    {
     int i=CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
     if (i==0) {
       copyXPMArea(0,65,9,18,7,39);
       copyXPMArea(32,65,11,11,6,5);
       FanOn(method);
       manual=1;
      } else {
       copyXPMArea(10,65,9,18,7,39);
       copyXPMArea(20,65,11,11,6,5);
       FanOff(method);
       manual=0;
      }     
    }
    break;
   }
  }
  usleep(250000L);
  i++;
  if (i>10)
  {
   i=0;
   if (FanStatus(method)==0x00) 
   {
    copyXPMArea(20,65,11,11,6,5);
    copyXPMArea(10,65,9,18,7,39); 
   }
    else
   {
    if (!manual) { copyXPMArea(44,65,11,11,6,5); copyXPMArea(0,65,9,18,7,39); }
   }
  }
 }
}

void usage(void) {
 fprintf(stderr, "\nwmfan+smp - version 1.0, by timecop@linuxwarez.com\n\n");
 fprintf(stderr, "usage:\n");
 fprintf(stderr, "\t-display <display name>\n");
 fprintf(stderr, "\t-h\tthis screen\n");
 fprintf(stderr, "\n");
}
